//Решение задач по заказу
function solvingCustomTask(messageServerIP,..//IP сервера обмена сообщениями
                           messageServerPort,..//Порт обмена сообщениями
                           dateMessagePointName,..//Точка обмена сообщениями, через которую обмениваемся получаем данными
                           sendOutputDateAdresatMessagePointName,..//Точка обмена сообщениями адресата, на которую мы отправляем ответ адресату
                           fNeedExit..//Функция необходимости выхода из сервера задач
                           )
    //Проверяем аргументы
    nArg = argn(2);//Число аргументов
    if nArg < 4 then
        error('This function solvingCustomTask takes 4 or 5 arguments');
    elseif (nArg == 4) || isempty(fNeedExit) then
        fNeedExit = NeedExitStandart;//Приравниваем стандартное условие выхода
    end
    if type(messageServerIP) ~= 10 then
        error('The solvingCustomTask function''s first argument must be string');
    end
    if (type(messageServerPort) ~= 10) &&..
       (isempty(messageServerPort) ||..
        (type(messageServerPort) ~= 1)) then
        error('The solvingCustomTask function''s second argument must be string or integer');
    end
    if type(dateMessagePointName) ~= 10 then
        error('The solvingCustomTask function''s third argument must be string');
    end
    if type(sendOutputDateAdresatMessagePointName) ~= 10 then
        error('The solvingCustomTask function''s fourth argument must be string');
    end
    
    //Создаем точки обмена сообщениями
    dateMessagePointNameo = dateMessagePointName + 'o';
    disp(CreateMessagePoint(dateMessagePointName,..
                            messageServerIP,..
                            messageServerPort)($:-1:1));
    disp(CreateMessagePoint(dateMessagePointNameo,..
                            messageServerIP,..
                            messageServerPort)($:-1:1));
    
    //Принимаем сообщения
    IntermediateData = [];//Сюда сохраняем промежуточные данные
    needExit = %f;//Необходимость выходить
    while ~needExit
        //Получаем сообщение
        disp('Waiting reseived messages:');//Добавляем информацию об ожидании сообщения
        msgs = ReceiveMessage(dateMessagePointName, %t);//Получаем сообщение
        disp('Reseived messages:');//Добавляем информацию о получении сообщения
        
        //Заносим сообщения в логи и выводим отчет на экран
        disp(msgs);
        
        //Проверяем необходимость выхода (и заодно чистим сообщения от лишних команд)
        [needExit,msgs]=fNeedExit(msgs);
        
        //Обрабатываем результаты
        outputDatas = [];//Выходные данные по всем задачам
        if ~isempty(msgs) then
            for msg=matrix(msgs,[1, size(msgs,'*')])//Решаем задачи
                //Решаем задачу и получаем выходные данные
                disp('Solving task');//Выводим сообщение о решении задачи и отправляем в отчет
                [outputData, IntermediateData] = evstr(msg);//Решаем задачу
                
                //Формируем строку ответа и добавляем в лог
                outputDatas = [outputDatas; convertMatrixDataToString(outputData)];
            end
        else
            outputDatas = 'There are not tasks; exit';//Пустая строка
            disp(outputDatas);//Выводим сообщение о решении задачи и отправляем в отчет
        end
        
        //Отправляем результаты адресату
        disp('Sending messages:');
        disp(SendMessage(dateMessagePointNameo,..
                         sendOutputDateAdresatMessagePointName,..
                         outputDatas)($:-1:1));
    end
    
    //Выводим сообщение о завершении решения задач по заказу
    disp(SendMessage(dateMessagePointNameo,..
                     sendOutputDateAdresatMessagePointName,..
                     'Exit from tasks server process')($:-1:1));
    
    //Закрываем точки обмена сообщениями
    disp(DestroyMessagePoint(dateMessagePointName)($:-1:1));
    disp(DestroyMessagePoint(dateMessagePointNameo)($:-1:1));
endfunction

//Функция перевода данных в строку
function outputData=convertMatrixDataToString(outputData)
    if type(outputData) == 10 then
        outputData = '\""' + string(outputData) + '\""';//Переводим в массив строк и добавляем кавычки
    else
        if isempty(outputData)
            outputData = 'Not answered data';//Переводим в строку (пустая строка)
        else
            outputData = string(outputData);//Переводим в массив строк
        end
    end
    if size(outputData,'*') > 1 then
        outputData = strcat(outputData, ', ', 'c');//Конкатенуем колонки
        outputData = strcat(outputData, '; ');//Конкатенуем колонки
        outputData = '\[' + outputData + '\]';//Добавляем квадратные скобки
    end
endfunction
